<?php
/*
+------------------------------------------------------------------------+
| ConPresso-Module "Track" File Injection Detection                      |
+------------------------------------------------------------------------+
| Copyright (c) Markus Rsel http://www.conpresso4.de                    |
+------------------------------------------------------------------------+
| LICENSE INFO                                                           |
| This source file is subject to ConPresso license version 2002-11-16    |
| that is bundled with this package in the files LICENSE_EN.TXT or       |
| LICENSE_DE.TXT, and is available through the world-wide-web at         |
|     http://www.conpresso.de/license/LICENSE_EN-2002-11-16.TXT or       |
|     http://www.conpresso.de/license/LICENSE_DE-2002-11-16.TXT          |
|                                                                        |
| If you did not receive a copy of the ConPresso license and are unable  |
| to obtain it through the world-wide-web, please send a note to         |
| <license@conpresso.de> so we can e-mail you a copy immediately.        |
+------------------------------------------------------------------------+
| MODULE LICENSE                                                         |
| This souce file is also subject to the module license available        |
| through the world-wide-web at                                          |
|    http://www.conpresso4.de/_rubric/index.php?rubric=Lizenzbedingungen |
| It is NOT freeware and must not be redistributed!                      |
+------------------------------------------------------------------------+
| Authors: Markus Rsel <mods@50n.de>                                    |
+------------------------------------------------------------------------+

Version 4.4 / January 2019

*/

if (!defined('CPO')) exit;

define('CPO_MOD_TRACK', $dbPrefix."_mod_track");

$translator->addTranslation('track', CPO_BASEDIR.$moduleDirectory.'/lang');

$GLOBALS['modules']['track'] = array(
    'directory' => $moduleDirectory,
    'caption'   => __('Track (file injection detection)'),
    'install'   => 'function: track_install',
    'activate'  => 'function: track_activate',
    'deactivate'=> 'function: track_deactivate',
    'uninstall' => 'function: track_uninstall '.$moduleDirectory,
    'manage'	=> 'function: track_manage',
	// settings
	'filetypes'		=> 'jpg;gif;png',
	'excludedDirs'	=> '_data',
	'maxFileSize'	=> 10000000,
	'notification'	=> 0,
	'recipient'		=> $cpoSystem['admin_email'],
	'reportItems'	=> 'all'
	);

// Funktionen **********************************************************
function track_install()
{
	$trackdb = new DB;
	$trackdb->query("INSERT INTO ".CPO_OPTION." (type, options, id) VALUES ('track', 0, '') ");
	$trackdb->query("CREATE TABLE ".CPO_MOD_TRACK." ( fs_file varchar(255) NOT NULL default '', checksum varchar(50) NOT NULL default '', date int(11) NOT NULL default '0', chmod int(3) NOT NULL default '0', PRIMARY KEY  (fs_file), KEY checksum (checksum,date) ) ");
    $GLOBALS['message'][] = __('The track module was successfully installed!');
	log_track();
}
function track_uninstall($moduleDirectory)
{
	$trackdb = new DB;
	$trackdb->query("DELETE FROM ".CPO_OPTION." WHERE type='track' ");
	$trackdb->query("DROP TABLE ".CPO_MOD_TRACK." ");
	$GLOBALS['message'][] = sprintf(__('The track module was successfully removed from the ConPresso system. Please remove the module directory <i>%s</i> to finally uninstall the track module.'),$moduleDirectory);
}
function track_activate()
{
    $GLOBALS['message'][] = __('The track module got activated.');
}

function track_deactivate()
{
    $GLOBALS['message'][] = __('The track module got deactivated.');
}

function track_manage()
{
	global $dbPrefix, $db, $k, $module_status, $newsletter_rubric_id;
	if(array_key_exists($k, $module_status) && $module_status[$k]!=MODULE_INACTIVE)  {
		echo '<a href="'.BASE_ABS.$GLOBALS['modules']['track']['directory'].'/edit.php"><img src="'.SKIN_URL.'pic_free.gif" title="'.__('check').'" alt="'.__('check').'"></a>';
		echo '<a href="'.BASE_ABS.$GLOBALS['modules']['track']['directory'].'/settings.php"><img src="'.SKIN_URL.'pic_settings.gif" title="'.__('settings').'" alt="'.__('settings').'"></a>';
		echo '<a href="'.BASE_ABS.$GLOBALS['modules']['track']['directory'].'/md5.php"><img src="'.SKIN_URL.'pic_version.gif" title="'.__('upgrade check').'" alt="'.__('update check').'"></a>';
	} else {
		echo '<img src="'.SKIN_URL.'pic_shade.gif" class="pic_" style="background: url('.SKIN_URL.'pic_free.gif);" alt="'.  __('not available').'" title="'.__('not available').'">';
		echo '<img src="'.SKIN_URL.'pic_shade.gif" class="pic_" style="background: url('.SKIN_URL.'pic_settings.gif);" alt="'.  __('not available').'" title="'.__('not available').'">';
		echo '<img src="'.SKIN_URL.'pic_shade.gif" class="pic_" style="background: url('.SKIN_URL.'pic_version.gif);" alt="'.  __('not available').'" title="'.__('not available').'">';
	}
}

function log_track()
{
	global $cpoSystem;
	@mail('kontakt@conpresso4.de','Das Tracking-Modul wurde unter '.BASE_ABS.' installiert','','From: '.$cpoSystem['admin_email']);
}

function trackExtension($fileName)  {
	return substr(strrchr($fileName,'.'),1);
}
function trackURL()  {
    global $cpoSystem;
    $auto_base_url = 'http://';
    if(isset($_SERVER['HTTPS']) && strtolower($_SERVER['HTTPS'])=='on') {
        $auto_base_url='https://';
    }
    $auto_base_url .= $_SERVER['HTTP_HOST'];
    $DOCROOT = $_SERVER['DOCUMENT_ROOT'];
    if (empty($DOCROOT)) {
        $DOCROOT = substr(
            str_replace(array('\\', '//'), '/', $_SERVER['PATH_TRANSLATED']), 
            0, 
            -(strlen($_SERVER['SCRIPT_NAME']))
        );
    }
    $auto_base_url .= substr(str_replace('\\', '/', __FILE__),  
                          strlen(str_replace('\\', '/', realpath($DOCROOT))), 
                          -strlen('mod_track/2common-module.inc.php'));

    if (preg_match('=auto/?=i', $cpoSystem['base_url']) || empty($cpoSystem['base_url'])) {
        $base_url = $auto_base_url;
    }
    if (substr($base_url, -1)!='/') $base_url .= '/';
    return $base_url;       // absulute base url
}

function listTrackFiles($dir,$update=false,$truncate=false,$log=false)  {
		# $dir -> das auszuwertende Verzeichnis
		# $update -> true fr Datenbankupdate
		# $truncate -> true fr das Lschen der Datenbank
		# $log -> true fr das Loggen des letzten Checks
	global $fileList,$activeModules;
	$filetypes = explode(";",$activeModules['track']['filetypes']);
	$excludedDirs = explode(";",$activeModules['track']['excludedDirs']);
	$trackdb = new DB;
	if ($truncate)  {
		$trackdb->query("TRUNCATE ".CPO_MOD_TRACK." ");
		$GLOBALS['message'][] = __('Database updated!');
	}
	$h = opendir($dir);
	while($file = readdir($h))  {
		$fs_file = $dir.'/'.$file;
		if ($fs_file != $dir."/hash.inc.php" AND $file != "." AND $file != ".." AND !is_dir($fs_file) AND !in_array(trackExtension($file),$filetypes) AND !in_array($file,$excludedDirs))  {
			
			$buffer = '';
			$handle = fopen ($fs_file, 'rb');
			if (filesize($fs_file)<=$activeModules['track']['maxFileSize']) $buffer = fread($handle, max(1,filesize($fs_file)));
			fclose ($handle);

			$trackmd5 = md5($buffer);
			$trackstamp = filemtime($fs_file);
			$trackperms = substr(decoct(fileperms($fs_file)),3);

			if ($update) $trackdb->query("REPLACE INTO ".CPO_MOD_TRACK." ( fs_file , checksum , date , chmod ) VALUES ( '".$fs_file."', '".$trackmd5."', '".$trackstamp."', '".$trackperms."' ) ");
			$fileList[] = $fs_file;
		}
		if ($fs_file != $dir."/hash.inc.php" AND $file != "." AND $file != ".." AND is_dir($fs_file) AND !in_array(trackExtension($file),$filetypes) AND !in_array($file,$excludedDirs))  {
			listTrackFiles($fs_file,$update,false,false);
		}
	}
	closedir($h);
	if ($log) $trackdb->query("UPDATE ".CPO_OPTION." SET options = '".time()."' WHERE type = 'track' ");
	if ($truncate && count($fileList)) $GLOBALS['message'][] = count($fileList) . ' '.__('database entries');
}

// Der automatische Check und Benachrichtigung ***********************************************
// Letzter Check war...
if(array_key_exists('track', $activeModules))  {
	$trackdb = new DB;
	$trackdb->query("SELECT options FROM ".CPO_OPTION." WHERE type='track' ");
	$trackdb->next_record();
	$trackLastUpdate = $trackdb->v('options');
	// Benachrichtigung aktiv und Zeit abgelaufen?
	if ($activeModules['track']['notification'] && time() >= $trackLastUpdate+$activeModules['track']['notification'])  {
		include(CPO_BASEDIR.$activeModules['track']['directory'].'/track.inc.php');
		$changes = trim(strip_tags ($trackOutput));
		if ($changes)  {
            // figure out and set base_url

            // hash for update-link
            $updateToken = substr(md5($changes),0,6);
            $updateHash = md5($updateToken);

            // Users online
			$users = array();
			$trackdb->query("SELECT user, ip, ip_name FROM ".CPO_LOG_LOGIN.", ".CPO_USER." WHERE user_id=".CPO_USER.".id ORDER BY user ");
			while($trackdb->next_record()) $users[] = $trackdb->v('user').' ('.$trackdb->v('ip_name').' ['.$trackdb->v('ip').'])';
			$changes = __('Please log in to check the changes')."\n".trackURL()."_admin/index.php\n\n".$changes."\n\n".__('Users online:')." ".implode(', ',$users);
            $changes .= "\n\n".__('If you are responsible for these changes and if you are sure that this is not an attack, you can confirm the changes and update the database by clicking the following link:')."\n\n".trackURL().'mod_track/update.php?hash='.$updateToken;

            file_put_contents(CPO_BASEDIR.$activeModules['track']['directory'].'/hash.inc.php',$updateHash);
			mail(
				$activeModules['track']['recipient'],
				'mod_track: '.__('filechanges detected'),
				$changes,
				'From: '.$cpoSystem['system_email']
				);
		}
	}
}

?>